==> hlog-0.20-rc1.tar.gz: hlog-0.20-rc1/hlog/contest/callout.c <==
/*
 * Copyright (c) 2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/mem.h>

#include <hlog/lcd.h>

#include "contest.h"

#define DSTR_MAX_LEN	100
#define CHAR_WIDTH	(LCD_CHAR_WIDTH + 1 /* space */)

/* index bits MSB to LSB: upper left, upper right, lower left, lower right */
static const char *blocks[16] = {
	[0x00] = " ",
	[0x01] = "\xe2\x96\x97", /* U+2597 - lower right */
	[0x02] = "\xe2\x96\x96", /* U+2596 - lower left */
	[0x03] = "\xe2\x96\x84", /* U+2584 - lower half */
	[0x04] = "\xe2\x96\x9d", /* U+259d - upper right */
	[0x05] = "\xe2\x96\x90", /* U+2590 - right half */
	[0x06] = "\xe2\x96\x9e", /* U+259e - upper right, lower left */
	[0x07] = "\xe2\x96\x9f", /* U+259f - upper right, lower half */
	[0x08] = "\xe2\x96\x98", /* U+2598 - upper left */
	[0x09] = "\xe2\x96\x9a", /* U+259a - upper left, lower right */
	[0x0a] = "\xe2\x96\x8c", /* U+258c - left half */
	[0x0b] = "\xe2\x96\x99", /* U+2599 - left half, lower right */
	[0x0c] = "\xe2\x96\x80", /* U+2580 - upper half */
	[0x0d] = "\xe2\x96\x9c", /* U+259c - upper half, lower right */
	[0x0e] = "\xe2\x96\x9b", /* U+259b - upper half, lower left */
	[0x0f] = "\xe2\x96\x88", /* U+2588 - full block */
};
#define PIXELSIZE 3

static WINDOW *win;
static size_t max_lcd_digits; /* max number of LCD digits we can display */
static size_t printable_width; /* printable width */
static size_t printable_height; /* printable height */
static char dstr[DSTR_MAX_LEN + 1]; /* printed string */

static void clear_callout(void)
{
	size_t row;

	for (row = 0; row < printable_height; row++)
		mvwprintw(win, row, 0, "%*s", printable_width, "");
}

static void show_lcd(void)
{
	const size_t dlen = strlen(dstr);
	const size_t llen = dlen * CHAR_WIDTH - 1;
	const size_t off = (printable_width - llen) / 2;
	bool bits[LCD_CHAR_HEIGHT + 1 /* blank */][printable_width + 1 /* \0 */];
	size_t row;
	size_t i, j;

	memset(bits, 0, sizeof(bits));

	/* convert text into bits */
	for (row = 0; row < LCD_CHAR_HEIGHT; row++) {
		for (i = 0; i < dlen; i++) {
			lcd_row(dstr[i], row, &bits[row][i * CHAR_WIDTH]);

			/* space between LCD chars */
			bits[row][(i + 1) * CHAR_WIDTH - 1] = false;
		}
	}

	for (row = 0; row < LCD_CHAR_HEIGHT; row += 2) {
		char txt[printable_width * PIXELSIZE + 1];

		/* convert bits into blocks to display */
		for (i = 0, j = 0; i < llen; i++) {
			const int idx = (bits[row + 0][i + 0] ? 0x08 : 0) |
					(bits[row + 0][i + 1] ? 0x04 : 0) |
					(bits[row + 1][i + 0] ? 0x02 : 0) |
					(bits[row + 1][i + 1] ? 0x01 : 0);
			const char *v = blocks[idx];

			strcpy(&txt[j], v);
			j += strlen(v);
		}

		txt[j] = '\0'; /* nul terminate */

		/* print the row */
		mvwprintw(win, (row / 2), off, "%s", txt);
	}
}

static void show(void)
{
	clear_callout();

	/* fall back to printing the string as is, if it is too long */
	if (strlen(dstr) > max_lcd_digits) {
		const size_t off = (printable_width - strlen(dstr)) / 2;

		mvwprintw(win, 0, off, "%s", dstr);
	} else {
		show_lcd();
	}
}

int callout_init(int height, int width, int row, int col)
{
	ASSERT(!win);

	printable_width = width;
	printable_height = height;
	max_lcd_digits = (printable_width + 1 /* trailing ws */) / CHAR_WIDTH;

	win = newwin(height, width, row, col);

	if (dstr[0]) {
		show();
		wrefresh(win);
	}

	return 0;
}

void callout_destroy(void)
{
	delwin(win);
}

void callout_set(const char *str)
{
	strcpy_safe(dstr, str, sizeof(dstr));

	if (!win)
		return;

	show();

	wrefresh(win);
}

==> hlog-0.22.tar.gz: hlog-0.22/hlog/contest/callout.c <==
/*
 * Copyright (c) 2021-2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/mem.h>

#include <hlog/lcd.h>

#include "contest.h"

#define DSTR_MAX_LEN	100
#define CHAR_WIDTH	(LCD_CHAR_WIDTH + 1 /* space */)

/* index bits MSB to LSB: upper left, upper right, lower left, lower right */
static const char *blocks[16] = {
	[0x00] = " ",
	[0x01] = "\xe2\x96\x97", /* U+2597 - lower right */
	[0x02] = "\xe2\x96\x96", /* U+2596 - lower left */
	[0x03] = "\xe2\x96\x84", /* U+2584 - lower half */
	[0x04] = "\xe2\x96\x9d", /* U+259d - upper right */
	[0x05] = "\xe2\x96\x90", /* U+2590 - right half */
	[0x06] = "\xe2\x96\x9e", /* U+259e - upper right, lower left */
	[0x07] = "\xe2\x96\x9f", /* U+259f - upper right, lower half */
	[0x08] = "\xe2\x96\x98", /* U+2598 - upper left */
	[0x09] = "\xe2\x96\x9a", /* U+259a - upper left, lower right */
	[0x0a] = "\xe2\x96\x8c", /* U+258c - left half */
	[0x0b] = "\xe2\x96\x99", /* U+2599 - left half, lower right */
	[0x0c] = "\xe2\x96\x80", /* U+2580 - upper half */
	[0x0d] = "\xe2\x96\x9c", /* U+259c - upper half, lower right */
	[0x0e] = "\xe2\x96\x9b", /* U+259b - upper half, lower left */
	[0x0f] = "\xe2\x96\x88", /* U+2588 - full block */
};
#define PIXELSIZE 3

static WINDOW *win;
static size_t max_lcd_digits; /* max number of LCD digits we can display */
static size_t printable_width; /* printable width */
static size_t printable_height; /* printable height */
static char dstr[DSTR_MAX_LEN + 1]; /* printed string */

static void clear_callout(void)
{
	size_t row;

	for (row = 0; row < printable_height; row++)
		mvwprintw(win, row, 0, "%*s", printable_width, "");
}

static void show_lcd(void)
{
	const size_t dlen = strlen(dstr);
	const size_t llen = dlen ? (dlen * CHAR_WIDTH - 1) : 0;
	const size_t off = (printable_width - llen) / 2;
	bool bits[LCD_CHAR_HEIGHT + 1 /* blank */][printable_width + 1 /* \0 */];
	size_t row;
	size_t i, j;

	memset(bits, 0, sizeof(bits));

	/* convert text into bits */
	for (row = 0; row < LCD_CHAR_HEIGHT; row++) {
		for (i = 0; i < dlen; i++) {
			lcd_row(dstr[i], row, &bits[row][i * CHAR_WIDTH]);

			/* space between LCD chars */
			bits[row][(i + 1) * CHAR_WIDTH - 1] = false;
		}
	}

	for (row = 0; row < LCD_CHAR_HEIGHT; row += 2) {
		char txt[printable_width * PIXELSIZE + 1];

		/* convert bits into blocks to display */
		for (i = 0, j = 0; i < llen; i++) {
			const int idx = (bits[row + 0][i + 0] ? 0x08 : 0) |
					(bits[row + 0][i + 1] ? 0x04 : 0) |
					(bits[row + 1][i + 0] ? 0x02 : 0) |
					(bits[row + 1][i + 1] ? 0x01 : 0);
			const char *v = blocks[idx];

			strcpy(&txt[j], v);
			j += strlen(v);
		}

		txt[j] = '\0'; /* nul terminate */

		/* print the row */
		mvwprintw(win, (row / 2), off, "%s", txt);
	}
}

static void show(void)
{
	clear_callout();

	/* fall back to printing the string as is, if it is too long */
	if (strlen(dstr) > max_lcd_digits) {
		const size_t off = (printable_width - strlen(dstr)) / 2;

		mvwprintw(win, 0, off, "%s", dstr);
	} else {
		show_lcd();
	}
}

int callout_init(int height, int width, int row, int col)
{
	ASSERT(!win);

	printable_width = width;
	printable_height = height;
	max_lcd_digits = (printable_width + 1 /* trailing ws */) / CHAR_WIDTH;

	win = newwin(height, width, row, col);

	if (dstr[0]) {
		show();
		wrefresh(win);
	}

	return 0;
}

void callout_destroy(void)
{
	delwin(win);
}

void callout_set(const char *str)
{
	strcpy_safe(dstr, str, sizeof(dstr));

	if (!win)
		return;

	show();

	wrefresh(win);
}
